/**
 *  File:         $$Workfile:$
 *  Created:      08-Feb-2004 16:01:16
 *  Author:       Inigo Surguy
 *  Last Changed: $$Date:$
 *
 *  Amendment History:
 *  $$Log$
 */
package net.surguy.logfilter;

import junit.framework.TestCase;

import java.io.StringWriter;

import org.apache.log4j.Category;
import org.apache.log4j.WriterAppender;
import org.apache.log4j.SimpleLayout;

/**
 * Test for LogPreloader - must be run with the JVM parameter -javaagent:net.surguy.logfilter.LogPreloader=debug
 * <p>
 * The last two tests are expected to fail with this version of ASM (1.4.1)
 *
 * @author Inigo Surguy
 * @created 08-Feb-2004 16:01:16
 */
public class LogPreloaderTest extends TestCase {
    StringWriter logMessages;
    DoesLogging doesLogging;

    public LogPreloaderTest(String s) { super(s); }

    protected void setUp() throws Exception {
        super.setUp();
        Category.getRoot().removeAllAppenders();
        logMessages = new StringWriter();
        Category.getRoot().addAppender(new WriterAppender(new SimpleLayout(), logMessages));
        doesLogging = new DoesLogging();
    }

    public void testSimpleLogAtDebug() throws Exception {
        doesLogging.simpleLogAtDebug();
        assertFalse("Log removal not worked at all - probably forgot to add instrumentation to loader with -javaagent:net.surguy.logfilter.LogPreloader", "DEBUG - simpleLogAtDebug".equals(logMessages.toString().trim()));
        assertEquals("", logMessages.toString().trim());
    }

    public void testSimpleLogAtInfo() throws Exception {
        doesLogging.simpleLogAtInfo();
        assertEquals("INFO - simpleLogAtInfo", logMessages.toString().trim());
    }

    public void testSimpleLogAtWarn() throws Exception {
        doesLogging.simpleLogAtWarn();
        assertEquals("WARN - simpleLogAtWarn", logMessages.toString().trim());
    }


    public void testDebugThenInfo() throws Exception {
        doesLogging.debugThenInfo();
        assertEquals("INFO - debugThenInfo", logMessages.toString().trim());
    }

    public void testReturnString() throws Exception {
        assertEquals("returnString", doesLogging.returnString());
        assertEquals("", logMessages.toString().trim());
    }

    public void testReturnStringWithInfo() throws Exception {
        assertEquals("returnStringWithInfo", doesLogging.returnStringWithInfo());
        assertEquals("INFO - returnStringWithInfo", logMessages.toString().trim());
    }

    public void testCategoryGetRoot() throws Exception {
        doesLogging.categoryGetRoot();
        assertEquals("INFO - categoryGetRoot", logMessages.toString().trim());
    }

    public void testCategoryGetRootViaInstance() throws Exception {
        assertEquals("root", doesLogging.categoryGetRootViaInstance());
        assertEquals("INFO - categoryGetRootViaInstance", logMessages.toString().trim());
    }

    public void testCategoryGetRootViaInstanceWithDebug() throws Exception {
        assertEquals("categoryGetRootViaInstanceWithDebug", doesLogging.categoryGetRootViaInstanceWithDebug());
        assertEquals("INFO - categoryGetRootViaInstanceWithDebug", logMessages.toString().trim());
    }

    public void testdoMathsBetweenDebugs() throws Exception {
        assertEquals(25, doesLogging.doMathsBetweenDebugs(5, 6));
        assertEquals("INFO - doMathsBetweenDebugs with 5 and 6", logMessages.toString().trim());
    }

    public void testDebugLogCallInsideLog() {
        doesLogging.debugLogCallInsideLog();
        assertEquals("", logMessages.toString().trim());
    }

    public void testInfoLogCallInsideLog() {
        doesLogging.infoLogCallInsideLog();
        assertEquals("INFO - Logger name is root.", logMessages.toString().trim());
    }


    public void testStaticInnerClass() {
        doesLogging.staticInnerClass();
        assertEquals("INFO - StaticInnerLogger", logMessages.toString().trim());
    }

    public void testMemberInnerClass() {
        doesLogging.memberInnerClass();
        assertEquals("INFO - MemberInnerLogger", logMessages.toString().trim());
    }

    // Fails
    public void testLocalClass() {
        doesLogging.localInnerClass();
        assertEquals("INFO - LocalInnerLogger", logMessages.toString().trim());
    }

    // Fails - a method is generated in the outer class to provide an accessor for the
    // inner class, and the log remover doesn't cope properly with it
    public void testAnonymousInnerClass() {
        doesLogging.anonymousInnerClass();
        assertEquals("INFO - AnonymousInnerLogger", logMessages.toString().trim());
    }



}
